
#include <linux/init.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/moduleparam.h>
#include "pll-lib.h"

int dvbpll_td1316_init(struct bus_i2c* bus, void* pll_data)
{
	struct dvbpll_td1316_data* pll = (struct dvbpll_td1316_data*) pll_data;
	static u8 disable_mc44BC374c[] = { 0x1d, 0x74, 0xa0, 0x68 };
	u8 pll_inittab[] = { 0x0b, 0xf5, 0x85, 0xab };
	int result;

	if (pll->atc) pll_inittab[2] = 0x88;

	if ((result = bus->write(bus->data, pll->mc44bc_addr, disable_mc44BC374c, sizeof(disable_mc44BC374c))) !=0) return result;
	if ((result = bus->write(bus->data, pll->pll_addr, pll_inittab, sizeof(pll_inittab))) !=0) return result;

	return 0;
}

int dvbpll_td1316_set(struct bus_i2c* bus, void* pll_data, struct dvb_frontend_parameters* fe_params)
{
	struct dvbpll_td1316_data* pll = (struct dvbpll_td1316_data*) pll_data;
	int cp;
	int band;
	int filter;
	int tuner_frequency;
	u8 tuner_buf[4];
	int result;

	// determine charge pump
	tuner_frequency = fe_params->frequency + 36130000;
	if (tuner_frequency < 87000000) {
		return -EINVAL;
	} else if (tuner_frequency < 130000000) {
		cp = 3;
	} else if (tuner_frequency < 160000000) {
		cp = 5;
	} else if (tuner_frequency < 200000000) {
		cp = 6;
	} else if (tuner_frequency < 290000000) {
		cp = 3;
	} else if (tuner_frequency < 420000000) {
		cp = 5;
	} else if (tuner_frequency < 480000000) {
		cp = 6;
	} else if (tuner_frequency < 620000000) {
		cp = 3;
	} else if (tuner_frequency < 830000000) {
		cp = 5;
	} else if (tuner_frequency < 895000000) {
		cp = 7;
	} else {
		return -EINVAL;
	}

	// determine band
	if (fe_params->frequency < 49000000) {
		return -EINVAL;
	} else if (fe_params->frequency < 159000000) {
		band = 1;
	} else if (fe_params->frequency < 444000000) {
		band = 2;
	} else if (fe_params->frequency < 861000000) {
		band = 4;
	} else {
		return -EINVAL;
	}

	// work out filter
	switch (fe_params->u.ofdm.bandwidth) {
	case BANDWIDTH_6_MHZ:
		filter = 0;
		break;

	case BANDWIDTH_7_MHZ:
		filter = 0;
		break;

	case BANDWIDTH_8_MHZ:
		filter = 1;
		break;

	default:
		return -EINVAL;
	}

	// calculate divisor
	// ((36130000+((1000000/6)/2)) + Finput)/(1000000/6)
	tuner_frequency =
	      (((fe_params->frequency / 1000) * 6) + 217280) / 1000;

	// setup tuner buffer
	tuner_buf[0] = tuner_frequency >> 8;
	tuner_buf[1] = tuner_frequency & 0xff;
	tuner_buf[2] = 0xca;
	tuner_buf[3] = (cp << 5) | (filter << 3) | band;

	// set it
	if ((result = bus->write(bus->data, pll->pll_addr, tuner_buf, sizeof(tuner_buf))) !=0) return result;
	msleep(1);
   
	return(0);
}


EXPORT_SYMBOL_GPL(dvbpll_td1316_init);
EXPORT_SYMBOL_GPL(dvbpll_td1316_set);

MODULE_DESCRIPTION("DVB PLL Library");
MODULE_LICENSE("GPL");
